*! version 1.5.1  03apr2020 [EM]

program define mkweights4, rclass

// Create weights for the Election data

// Syntax:
//
//   mkweights4 <spec> [if] [in] [, over(<varlist>) wgtvars(<varlist>) 
//     basewgt(<varname>) marginsfile(<filename>) strata(<varlist>) 
//     reps(<replications>) seed(<seed>) maxrep(integer) n(integer) * ]
//
// where <spec> is of the form <stub>* and <stub> is the beginning of
// the desired weight variable names.
// The optional "over" argument defines a set of categorical variables
// that define cells within which the weights should be computed separately
// wgtvars gives the list of variables to weight on; this should be a
//   subset of the totals available in the file wgtmargins.ster
// basewgt is the base weight; if omitted, 1 is used for all cases
// marginsfile : a .ster file containing the marginal distributions that
//   we need to weight against (if omitted, "wgtmargins.ster" is used)
// strata : strata for svyset (In the 2016 file, this was always _one,
//   i.e., 1, i.e., only one stratum; I don't remember why; maybe presence
//   of strata was required by one of the raking programs.) If omitted,
//   1 is used for all cases.
// reps : number of replication weights; default is 80
// seed : random seed for the replication weights; default is 90089
// maxrep : maximum number of iterations in the raking (default is 2000)
// n : bsweights option for subsampling; see help bsweights; default = -1
//
// * : trimmed_weight options; see trimmed_weight.ado for these options
//
// For example:
//   mkweights4 w*
// or
//   mkweights4 w20160712_* if (inrange(svydate,date("07/06/2016","MDY"),date("07/12/2016","MDY")))
// or
//   mkweights4 wgt_w02_* if (svywave == 2)
// or
//   mkweights4 wgt_w02_* , over(svyweek)
//
// In order to avoid potential confusion or nonuniqueness, it is
// better to make sure the stub does not end with a number.
// Also, the "if/in" should not restrict the sample by variables that
// are used in the weighting itself.

syntax anything [if] [in] [ , over(varlist) wgtvars(varlist) ///
    basewgt(varname) marginsfile(string) strata(varlist) reps(integer 80) ///
    seed(string) maxrep(integer 2000) n(integer -1) * ]

marksample touse, novarlist strok

// Trim spaces
local stubstar = trim("`anything'")
// Find and strip the star at the end
local starpos = strpos("`stubstar'","*")
local l = length("`stubstar'")
if (`starpos' != `l') {
    display as error "Incorrect variable specification"
    exit 198
}
local stub = subinstr("`stubstar'","*","",1)
local l : word count `stub'
if (`l' != 1) {
    display as error "Incorrect variable specification"
    exit 198
}

// So now stub is one "word", which is the start of the variable names

// Base weight
if ("`basewgt'" == "") {
    // No basewgt given
    quietly generate byte _one = 1 if (`touse')
    local basewgt2 _one
}
else {
    // basewgt given; the goal here is to set the basewgt to missing if
    // case is not to be used (not sure if this is the best way to proceed)
    quietly clonevar _bwgt = `basewgt' if (`touse')
    local basewgt2 _bwgt
}

// Read control matrices from file
// cmatlist is a list of control matrices, i.e., matrices that contain the
// target distributions of the variables that are used for weighting.
if ("`marginsfile'" == "") {
    local marginsfile "wgtmargins.ster"
}
estimates use `"`marginsfile'"'
local cmatlist `e(cmatlist)'
foreach c in `cmatlist' {
    matrix `c' = e(`c')
}
// Population size
local popsize = e(N)

// Strata for svyset
if ("`strata'" == "") {
    capture confirm variable _one
    if (_rc != 0) {
        // _one does not exist yet
        quietly generate byte _one = 1 if (`touse')
    }
    local strata _one
}

// Random seed for replication weights
if ("`seed'" == "") {
    local seed 90089
}

// Create weight

local totlist
local vlist
tempname val
foreach c in `cmatlist' {
    // variable name
    local v : rownames `c'

    if ("`wgtvars'" == "") {
        // All variables used
        local inwlist = 1
    }
    else {
        // Is variable in requested weight variable list?        
        local inwlist : list v in wgtvars
    }
    if (`inwlist' == 1) {
        local vlist `vlist' `v'
        // variable containing the control total
        local cnames : colnames `c'
        tempvar tot_`v'
        local totlist `totlist' `tot_`v''
        quietly {
            generate double `tot_`v'' = .
            local j = 0
            foreach cat in `cnames' {
                local ++j
                scalar `val' = `c'[1,`j']
                replace `tot_`v'' = `val' if (`v' == `cat')
            }
        }
    }
}

if ("`over'" == "") {
    // Create main weight
    trimmed_weight `basewgt2', by(`vlist') totvars(`totlist') generate(`stub'main) ///
        maxrep(`maxrep') `options'

    // Create replication weights
    svyset uasid [pw=`basewgt2'], strata(`strata')
    if (`reps' > 0 & !missing(`reps')) {
        bsweights `stub', reps(`reps') n(`n') seed(`seed') double ///
            calibrate(trimmed_weight @, by(`vlist') totvars(`totlist') replace ///
            maxrep(`maxrep') `options' )
    }
}
else {
    // Weight within groups
    tempvar grp
    quietly {
        egen long `grp' = group(`over')
    }

    // Create main weight
    overwgt4 `basewgt2', over(`grp') by(`vlist') totvars(`totlist') ///
        generate(`stub'main) maxrep(`maxrep') `options'

    // Create replication weights
    svyset uasid [pw=`basewgt2'], strata(`strata')
    if (`reps' > 0 & !missing(`reps')) {
        bsweights `stub', reps(`reps') n(`n') seed(`seed') double ///
            calibrate(overwgt4 @, over(`grp') by(`vlist') totvars(`totlist') replace ///
            maxrep(`maxrep') `options' )
    }
}

label variable `stub'main "Sampling weight"
local bsrlist
if (`reps' > 0 & !missing(`reps')) {
    forvalues r=1/`reps' {
        label variable `stub'`r' "Svy bootstrap repl.wgt `r'"
        local bsrlist `bsrlist' `stub'`r'
    }
    // Declare the replication weights so Stata can use them
    svyset uasid [pw=`stub'main], vce(bootstrap) bsrweight(`bsrlist')
}
else {
    // No replication weights; use standard asymptotic (linearized) s.e.'s
    svyset uasid [pw=`stub'main], vce(linearized)
}

// Drop temporary variables, if they were needed
capture drop _one
capture drop _bwgt

// Return population size
return scalar popsize = `popsize'

end

// ----------------------------------------------------------------


exit

// EOF
