*! version 1.0.0  26apr2020  [EM]

program define poststrat

syntax varlist [if] [in], poststrata(varname) postweight(varname) ///
    GENerate(namelist) RELative

marksample touse, novarlist strok

// Poststratify:
//   varlist is a list of weight variables
//   poststrata is a stratum variable (a single categorical variable),
//       which indicates the poststrata
//   postweight is a variable that has the population sizes of the strata
//       (so this should be constant within poststrata)
//   generate contains a list of new variable names corresponding with
//       varlist, which will be generated as poststratified weight variables
//   relative : rescale weights to have mean 1

local wlist   `varlist'
local newlist `generate'

// Values corresponding to the strata
quietly levelsof `poststrata' if (`touse'), local(pstrata)
local nstrata : word count `pstrata'
tempname stratasizes rmin rmax
matrix `stratasizes' = J(1, `nstrata', .)

// Loop over strata
tempname totpop
scalar `totpop' = 0
forvalues s=1/`nstrata' {
    // Value of the variable indicating the stratum
    local sval : word `s' of `pstrata'
    // Stratum size
    quietly summarize `postweight' if (`touse' & `poststrata' == `sval')
    scalar `rmin' = r(min)
    scalar `rmax' = r(max)
    if (`rmin' != `rmax') {
        display as error _n "!!! Error !!! Stratum size not constant within stratum"
        display as error "`poststrata' = `sval': min = " `rmin' "; max = " `rmax' _n
    }
    matrix `stratasizes'[1, `s'] = `rmin'
    scalar `totpop' = `totpop' + `rmin'
}

tempname stats popsize rsum rmean rsd deff

local wno 0
foreach w of varlist `wlist' {
    local ++wno
    local wnew : word `wno' of `newlist'
    quietly generate double `wnew' = .
    label variable `wnew' "Poststratified weight based on `w'"

    matrix `stats' = J(`nstrata'+1,7,.)

    forvalues s=1/`nstrata' {
        // Value of the variable indicating the stratum
        local sval : word `s' of `pstrata'
        scalar `popsize' = `stratasizes'[1,`s']

        // Sum of the weights in the stratum
        quietly summarize `w' if (`touse' & `poststrata' == `sval')
        local nn = r(N)
        scalar `rsum'    = r(sum)
        scalar `rmean'   = r(mean)
        scalar `rsd'     = r(sd)
        scalar `deff'    = (`rsd')^2/(`rmean')^2 + 1

        matrix `stats'[`s',1] = `sval'
        matrix `stats'[`s',2] = `popsize'
        matrix `stats'[`s',3] = `nn'
        matrix `stats'[`s',4] = `rsum'
        matrix `stats'[`s',5] = `rmean'
        matrix `stats'[`s',6] = `rsd'
        matrix `stats'[`s',7] = `deff'

        // Adjust weight variable
        quietly replace `wnew' = `w' * `popsize'/`rsum' ///
            if (`touse' & `poststrata' == `sval')

    } // End loop over strata

    local s = `nstrata' + 1
    quietly summarize `wnew' if (`touse')
    local nn = r(N)
    scalar `rsum'    = r(sum)
    scalar `rmean'   = r(mean)
    scalar `rsd'     = r(sd)
    scalar `deff'    = (`rsd')^2/(`rmean')^2 + 1

    matrix `stats'[`s',1] = .
    matrix `stats'[`s',2] = `totpop'
    matrix `stats'[`s',3] = `nn'
    matrix `stats'[`s',4] = `rsum'
    matrix `stats'[`s',5] = `rmean'
    matrix `stats'[`s',6] = `rsd'
    matrix `stats'[`s',7] = `deff'

    display as text _n "Statistics for weight variables `w' and `wnew'" _n
    matlist `stats', format(%15.4fc)

    // Rescale to have mean 1 if requested
    if ("`relative'" != "") {
        quietly replace `wnew' = `wnew'/`rmean'
    }

} // End loop over wlist

end
exit

// EOF
