*! version 0.5.1  6apr2020 [MA-EM]

program define trimmed_weight, rclass

// Create trimmed weights

// Original code: Marco Angrisani
// do->ado, adaptation, options: Erik Meijer

// Syntax:
//
//   trimmed_weights basewgt [if] [in] , by(raking_vars) totvars(totals)
//     [ generate(raked_weight) replace maxrep(integer)
//       maxiter(integer) t_min(threshold) t_max(threshold)
//       p_min(threshold) p_max(threshold) notrim ptrim
//     ]
//
// basewgt is the base weight
// by lists the variables that are used in the raking (gender, race, etc.)
// totvars has the variables that contain the totals (number of men/women
//   in the population etc.)
// generate : name of the raked weight variable (if it's a new variable)
// replace : overwrite the basewgt with the raked weight
//   either generate or replace should be given
// maxrep : maximum number of iterations in the raking (default is 100)
//
// See
//   help survwgt
// for more details on these options.
//
// maxiter : max number of iterations of trimming (default = 50)
// t_min   : min relative weight for trimming (default = 0.25)
// t_max   : max relative weight for trimming (default = 4)
// p_min   : min relative weight for trimming, as percentile of the
//           untrimmed relative weight distribution (default = 10;
//           must be an integer)
// p_max   : max relative weight for trimming, as percentile of the
//           untrimmed relative weight distribution (default = 90;
//           must be an integer)
// notrim  : do not trim weights
// ptrim   : trim based on percentiles (implied by p_min and p_max)
//
// These options are partly mutually incompatible. They are checked as
// follows:
// 1. If notrim is given, no trimming is done.
// 2. If ptrim is given (but not notrim), trimming is done based on
//    percentiles.
// 3. If neither notrim nor ptrim are given, trimming is done on fixed
//    relative weight thresholds, unless t_min and t_max are not given
//    and p_min or p_max is given. That is, if the only trimming info is
//    p_min or p_max, percentile trimming is done.
// 4. Once it has been decided whether to trim, and if so, whether to
//    use fixed thresholds or percentile thresholds, defaults for the
//    thresholds are used if the thresholds have not been explicitly given.

syntax varname [if] [in] , by(varlist) totvars(varlist) ///
    [ generate(name) replace maxrep(integer 100)        ///
      maxiter(integer 50) t_min(string) t_max(string)   ///
      p_min(string) p_max(string) noTRIM ptrim ]

marksample touse, novarlist strok

local basewgt      `varlist'
local var_list     `by'
local tot_var_list `totvars'

// Name of the new weight variable (or old if replace)
if ("`generate'" == "") {
    local newwgt `basewgt'
}
else {
    if ("`replace'" != "") {
        display as error "!!! Error !!! In trimmed_weight, choose either generate or replace, not both"
        display as error "generate(`generate') used to store the raked weights and replace ignored"
    }
    local newwgt `generate'
    quietly generate double `newwgt' = .
}

// Trimming

tempname trmin trmax
if ("`trim'" == "notrim") {
    // No trimming
    local trimming   0
    local ptrimming  0
    scalar `trmin' = 0
    scalar `trmax' = .
}
else {
    // Trimming
    local trimming 1

    if ("`ptrim'" != "") {
        // Percentile trimming
        local ptrimming 1
    }
    else if ("`p_min'" == "" & "`p_max'" == "") {
        // Fixed threshold trimming
        local ptrimming 0
    }
    else if ("`t_min'" != "" | "`t_max'" != "") {
        // Fixed threshold trimming takes precedence over percentile trimming
        local ptrimming 0
    }
    else {
        // p_min or p_max are given and no fixed threshold given, so:
        // percentile trimming
        local ptrimming 1
    }

    if (`ptrimming') {
        // Percentile trimming; check/set percentiles
        if ("`p_min'" != "") {
            local prmin = `p_min'
        }
        else {
            local prmin = 10  // Default if not given
        }
        if ("`p_max'" != "") {
            local prmax = `p_max'
        }
        else {
            local prmax = 90  // Default if not given
        }
        scalar `trmin' = 0
        scalar `trmax' = .
    }
    else {
        // Threshold trimming; check/set thresholds
        if ("`t_min'" != "") {
            scalar `trmin' = `t_min'
        }
        else {
            scalar `trmin' = 0.25  // Default if not given
        }
        if ("`t_max'" != "") {
            scalar `trmax' = `t_max'
        }
        else {
            scalar `trmax' = 4.00  // Default if not given
        }
    }
}

// Clone with if, to accommodate weighting part of the sample.
// (survwgt does not allow if/in, so need to set basewgt to missing
// to accomplish that)

tempvar basewgt2
quietly clonevar `basewgt2' = `basewgt' if (`touse')

tempvar weight rel_weight flag

// Use survwgt rake to generate initial weights
survwgt rake `basewgt2', by(`var_list') totvars(`tot_var_list') ///
    generate(`weight') maxrep(`maxrep') check(1)

quietly {
    summarize `weight' if (`touse'), meanonly
    generate double `rel_weight' = `weight'/r(mean) if (`touse')

    if (!`trimming') {
        // No trimming; return results and exit

        // Copy final weight into permanent weight variable
        replace `newwgt' = `rel_weight' if (`touse')
        count if (`touse')
        local nn = r(N)

        // Return results: number of iterations, whether converged, final weight
        // variable, and settings
        return scalar maxrep    = `maxrep'
        return scalar N         = `nn'
        return local  totvars `tot_var_list'
        return local  by      `var_list'
        return local  weight  `newwgt'
        return local  basewgt `basewgt'

        exit
    }

    if (`ptrimming') {
        // Compute the fixed thresholds that correspond with the percentiles
        summarize `rel_weight' if (`touse'), detail
        scalar `trmin' = r(p`prmin')
        scalar `trmax' = r(p`prmax')
    }

    // Number of cases with out-of-bounds weights
    generate byte `flag' = (`touse' & (`rel_weight' < `trmin' | `rel_weight' > `trmax') ///
                            & !missing(`rel_weight'))
    count if (`flag' == 1)
    local noutbound = r(N)
}

// --------------------------------------------------------------------
// RAKING ALGORITHM IMPLEMENTATION: ITERATIONS > 0
// --------------------------------------------------------------------

local i = 0                     // Iteration number
local done = (`noutbound' == 0) // Whether done with iterating

tempvar usvar1 usvar2 usvar3 diff reldiff
quietly {
    generate double `usvar1'  = .
    generate double `usvar2'  = .
    generate double `usvar3'  = .
    generate double `diff'    = .
    generate double `reldiff' = .
}
tempname sum_diff add msreldiff

while (`i' < `maxiter' & !`done') { // Loop over iterations
    local ++i

    display "Loop # `i'"

    // Trimming

    quietly {

        // Censor to allowed bounds
        replace `usvar1' = `rel_weight' if (`touse' & `flag' == 0)
        replace `usvar1' = `trmax' if (`touse' & !missing(`rel_weight') & `rel_weight' > `trmax')
        replace `usvar1' = `trmin' if (`touse' & !missing(`rel_weight') & `rel_weight' < `trmin')

        // How many cases are within bounds?
        count if (`touse' & `flag' == 0)
        local ninbound = r(N)

        // Lost weight from out of bounds cases
        replace `diff' = `rel_weight' - `usvar1' if (`touse' & `flag' == 1)
        replace `diff' = .                       if (`touse' & `flag' == 0)
        summarize `diff' if (`touse' & `flag' == 1), meanonly
        scalar `sum_diff' = r(sum)

        // Redistributing lost weight
        scalar `add' = `sum_diff'/`ninbound'
        replace `usvar2' = `usvar1'         if (`touse' & `flag' == 1)
        replace `usvar2' = `usvar1' + `add' if (`touse' & `flag' == 0)

        // Raking
        capture drop `weight'
        survwgt rake `usvar2', by(`var_list') totvars(`tot_var_list') ///
            generate(`weight') maxrep(`maxrep') check(1)

        summarize `weight' if (`touse'), meanonly
        replace `usvar3' = `weight'/r(mean) if (`touse')

        // Mean square relative difference between old and new weight
        replace `reldiff' = ((`usvar3' - `rel_weight')^2)/`rel_weight' if (`touse')
        summarize `reldiff' if (`touse'), meanonly
        scalar `msreldiff' = r(mean)
        noisily display as text "Mean relative change in rel_weight = " `msreldiff'

        replace `rel_weight' = `usvar3' if (`touse')

        // Number of cases with out-of-bounds weights
        replace `flag' = (`touse' & (`rel_weight' < `trmin' | `rel_weight' > `trmax') ///
                                  & !missing(`rel_weight'))
        count if (`flag' == 1)
        local noutbound = r(N)

        local done = (`noutbound' == 0) // Whether done with iterating

    } // end quietly
} // end iteration loop

// Copy final weight into permanent weight variable
quietly replace `newwgt' = `rel_weight' if (`touse')
quietly count if (`touse')
local nn = r(N)

// Return results: number of iterations, whether converged, final weight
// variable, and settings
return scalar converged = `done'
return scalar iter      = `i'
return scalar t_max     = `trmax'
return scalar t_min     = `trmin'
return scalar maxiter   = `maxiter'
return scalar maxrep    = `maxrep'
return scalar N         = `nn'
return local  totvars `tot_var_list'
return local  by      `var_list'
return local  weight  `newwgt'
return local  basewgt `basewgt'

end
exit

// EOF
