// Template for creating weights for custom samples for the
// USC Dornsife 2020 election poll.

// The samples must be intended to be nationally representative, so a
// typical application would be a custom survey period or a wave, but
// not, say, selection on demographics.

// NOTE!!! You need to have the survwgt package installed in Stata.
// If you don't have it (or are unsure), type
//   findit survwgt
// In the resulting screen you should be able to install the package
// with a few clicks.

// This command implies that Stata searches in the current directory
// first for ado files. If you put the .ado files somewhere else than
// in the current directory, change this command accordingly.

adopath ++.

// ----
// Step 1: Load data file

use "fulldata.dta", clear

// ----
// Step 2: Define sample

// Note: In the poll, we only keep the first interview of the same person
// on the same day. The code also excludes cases with inconsistent answers
// (probabilities don't add up to 100) and cases where one or more of
// the weight variables are missing, but these are minor adjustments.

// Replace with your specification, but note that the selection must still
// be intended to be a representative sample of the population of all U.S.
// citizens age 18 and over. If you want to do analyses of demographic
// subgroups, you'd first weight the whole sample and in the estimation
// command (e.g., mean) define the subpopulation.

generate byte mysample = (pollwave == 1)  
// the example provided here would create weights for the first wave of the poll, 
// or you could create a unique set of responses by identifying the last 
// response from every respondent in the poll.  If you want to create a set of 
// longitudinal weights (e.g. weekly or by wave) use "Step 4 alternative" 
// below.   

// ----
// Step 3: Define variables you want to match on in the weighting, or use the defaul set below  

// Variables that are available:
//   pr_female        Gender
//   pr_race4         Race-ethnicity (4 categories)
//   pr_race5         Race-ethnicity (5 categories)
//   pr_age_cat3      Age (3 categories)
//   pr_edu3          Education (3 categories)
//   pr_urban3        Census Urbanicity (rural, mixed, urban)
//   pr_statereside   State (50 + DC)
//   pr_region7       National Regions (7 categories)
//   pr_vote2016      Vote in 2016 (4 categories)

// This is the default list in the poll:

local wgtvars pr_female pr_race5 pr_age_cat3 pr_edu3 pr_urban3 pr_region7 ///
    pr_vote2016

// ----
// Step 4: Create the weights

// See mkweights4.ado for available options. These are the ones we use
// in the poll graphics. Most users would not change these options.

quietly mkweights4 w_* if (mysample == 1), wgtvars(`wgtvars') ///
    basewgt(pr_basewgt) marginsfile(`"cps_marg_2020.ster"') reps(0) ///
    maxrep(100) maxiter(50) p_min(10) p_max(90)

// This has created a weight variable called w_main, which you can rename
// if you want. Change subsequent code accordingly.


// ----
// Step 5: svyset the data

svyset uasid [pw=w_main], vce(linearized)

// After this, any "svy:" command takes the weights and clustering into
// account.

// --------------------------------------------------------------------
// ALTERNATIVE: weights by over() groups

// The over() option weights the sample within each category of the over
// variable, e.g., it weights each week separately. Analogous to the sample
// definition above, this only makes sense if the sample is intended to
// be nationally representative within each category of the over variable,
// so this will be limited to survey periods or waves.

// ----
// Step 4-alt: Create the weights

// Uncomment and adapt to your own specification (if so, comment out the
// version in Step 4 above).  This specification will run 

// in the example below, the weights would be created for each pollwave.  This
// would create a set of weighted full-sample waves.   

// quietly mkweights4 w2_* , wgtvars(`wgtvars') over(pollwave) ///
//     basewgt(pr_basewgt) marginsfile("cps_marg_2020.ster") reps(0) ///
//     maxrep(100) maxiter(50) p_min(10) p_max(90)

// This has created a weight variable called w2_main, which you can rename
// if you want. Change subsequent code accordingly.

// ----
// Step 5-alt: svyset the data

// With the over() option, the over variable (pollweek in the example)
// needs to be declared as a poststratum.

// Uncomment and adapt the following two commands your own specification
// (if so, comment out the version in Step 5 above).

// generate double tot_pop = e(N) // = 231588121.4598172
// svyset uasid [pw=w2_main], vce(linearized) poststrata(pollweek) postweight(tot_pop)

// --------------------------------------------------------------------
// ESTIMATION

// Now, you can use estimation commands with the "svy:" prefix. They
// take the weights and other settings from the svyset into account. See
//   help svy
// and/or Stata's SVY manual for available options and commands;
//   svy: mean
// and
//   svy: ratio
// are the ones we use in the poll.
//   svy: tabulate
// is also likely useful.
